{% extends "account/base.html" %}

{% load i18n %}

{% block head_title %}{% trans "Confirm Access" %}{% endblock head_title %}

{% block content %}

<h1>{% trans "Confirm Access" %}</h1>

<p>{% blocktranslate %}To safeguard the security of your account, please enter your password:{% endblocktranslate %}</p>

<form class="reauthenticate" method="POST" action="{% url 'account_reauthenticate' %}">
  {% csrf_token %}
  {{ form.as_p }}
  {% if redirect_field_value %}
  <input type="hidden" name="{{ redirect_field_name }}" value="{{ redirect_field_value }}" />
  {% endif %}
  <button type="submit">{% trans "Confirm" %}</button>
</form>

{% endblock content %}
