# Generated by Django 4.2.4 on 2023-08-23 18:17

import django.db.models.functions.text
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("account", "0004_alter_emailaddress_drop_unique_email"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="emailaddress",
            index=models.Index(
                django.db.models.functions.text.Upper("email"),
                name="account_emailaddress_upper",
            ),
        ),
    ]
